@@ -1,4 +1,4 @@
-#!/usr/bin/perl
+#!/usr/bin/perl -w
 eval 'exec perl -S $0 "$@"'
     if 0;
 
@@ -6,7 +6,7 @@ eval 'exec perl -S $0 "$@"'
 
 pflogsumm.pl - Produce Postfix MTA logfile summary
 
-Copyright (C) 1998-2003 by James S. Seymour, Release 1.1.0.
+Copyright (C) 1998-2007 by James S. Seymour, Release 1.1.1.
 
 =head1 SYNOPSIS
 
@@ -326,10 +326,11 @@ Copyright (C) 1998-2003 by James S. Seymour, Release 1.1.0.
 
 =head1 REQUIREMENTS
 
-    Pflogsumm requires the Date::Calc module, which can be obtained from
-    CPAN at http://www.perl.com.
+    For certain options (e.g.: --smtpd_stats), Pflogsumm requires the
+    Date::Calc module, which can be obtained from CPAN at
+    http://www.perl.com.
 
-    Pflogsumm is currently written and tested under Perl 5.005_03.
+    Pflogsumm is currently written and tested under Perl 5.8.3.
     As of version 19990413-02, pflogsumm worked with Perl 5.003, but
     future compatibility is not guaranteed.
 
@@ -358,12 +359,11 @@ Copyright (C) 1998-2003 by James S. Seymour, Release 1.1.0.
 use strict;
 use locale;
 use Getopt::Long;
-# ---Begin: SMTPD_STATS_SUPPORT---
-use Date::Calc qw(Delta_DHMS);
-# ---End: SMTPD_STATS_SUPPORT---
+eval { require Date::Calc };
+my $hasDateCalc = $@ ? 0 : 1;
 
 my $mailqCmd = "mailq";
-my $release = "1.1.0";
+my $release = "1.1.1";
 
 # Variables and constants used throughout pflogsumm
 use vars qw(
@@ -504,6 +504,32 @@ if(defined($opts{'version'})) {
     exit 0;
 }
 
+if($hasDateCalc) {
+    # manually import the Date::Calc routine we want
+    #
+    # This looks stupid, but it's the only way to shut Perl up about
+    # "Date::Calc::Delta_DHMS" used only once" if -w is on.  (No,
+    # $^W = 0 doesn't work in this context.)
+    *Delta_DHMS = *Date::Calc::Delta_DHMS;
+    *Delta_DHMS = *Date::Calc::Delta_DHMS;
+
+} elsif(defined($opts{'smtpdStats'})) {
+    # If user specified --smtpd_stats but doesn't have Date::Calc
+    # installed, die with friendly help message.
+     die <<End_Of_HELP_DATE_CALC;
+
+The option "--smtpd_stats" does calculations that require the
+Date::Calc Perl module, but you don't have this module installed.
+If you want to use this extended functionality of Pflogsumm, you
+will have to install this module.  If you have root privileges
+on the machine, this is as simple as performing the following
+command:
+
+     perl -MCPAN -e 'install Date::Calc'
+
+End_Of_HELP_DATE_CALC
+}
+
 $dateStr = get_datestr($opts{'d'}) if(defined($opts{'d'}));
 
 # debugging
@@ -516,8 +542,8 @@ while(<>) {
     my $logRmdr;
     next unless((($msgMonStr, $msgDay, $msgHr, $msgMin, $msgSec, $logRmdr) =
 	/^(...) +(\d+) (..):(..):(..) \S+ (.+)$/o) == 6);
-    unless((($cmd, $qid) = $logRmdr =~ m#^(?:vmailer|postfix|$syslogName)/([^\[:]*).*?: ([^:\s]+)#o) == 2 ||
-           (($cmd, $qid) = $logRmdr =~ m#^((?:vmailer|postfix)(?:-script)?)(?:\[\d+\])?: ([^:\s]+)#o) == 2)
+    unless((($cmd, $qid) = $logRmdr =~ m#^(?:postfix|$syslogName)/([^\[:]*).*?: ([^:\s]+)#o) == 2 ||
+           (($cmd, $qid) = $logRmdr =~ m#^((?:postfix)(?:-script)?)(?:\[\d+\])?: ([^:\s]+)#o) == 2)
     {
 	#print UNPROCD "$_";
 	next;
@@ -547,16 +573,16 @@ while(<>) {
 	$rejRmdr =~ s/( from \S+?)?; from=<.*$//o unless($opts{'verbMsgDetail'});
 	$rejRmdr = string_trimmer($rejRmdr, 64, $opts{'verbMsgDetail'});
 	if($rejSubTyp eq "reject") {
-	    ++$rejects{$cmd}{$rejReas}{$rejRmdr};
+	    ++$rejects{$cmd}{$rejReas}{$rejRmdr} unless($opts{'noRejectDetail'});
 	    ++$msgsRjctd;
 	} elsif($rejSubTyp eq "warning") {
-	    ++$warns{$cmd}{$rejReas}{$rejRmdr};
+	    ++$warns{$cmd}{$rejReas}{$rejRmdr} unless($opts{'noRejectDetail'});
 	    ++$msgsWrnd;
 	} elsif($rejSubTyp eq "hold") {
-	    ++$holds{$cmd}{$rejReas}{$rejRmdr};
+	    ++$holds{$cmd}{$rejReas}{$rejRmdr} unless($opts{'noRejectDetail'});
 	    ++$msgsHld;
 	} elsif($rejSubTyp eq "discard") {
-	    ++$discards{$cmd}{$rejReas}{$rejRmdr};
+	    ++$discards{$cmd}{$rejReas}{$rejRmdr} unless($opts{'noRejectDetail'});
 	    ++$msgsDscrdd;
 	}
 	++$rejPerHr[$msgHr];
@@ -617,7 +643,6 @@ while(<>) {
 				  \${$msgsPerDay{$revMsgDateStr}}[4]);
 	    }
 	}
-# ---Begin: SMTPD_STATS_SUPPORT---
 	else {
 	    next unless(defined($opts{'smtpdStats'}));
 	    if($logRmdr =~ /: connect from /o) {
@@ -658,7 +683,6 @@ while(<>) {
 		}
 	    }
 	}
-# ---End: SMTPD_STATS_SUPPORT---
     } else {
 	my $toRmdr;
 	if((($addr, $size) = $logRmdr =~ /from=<([^>]*)>, size=(\d+)/o) == 2)
@@ -696,7 +720,7 @@ while(<>) {
 	    }
 	}
 	elsif((($addr, $relay, $delay, $status, $toRmdr) = $logRmdr =~
-		/to=<([^>]*)>, (?:orig_to=<[^>]*>, )?relay=([^,]+), delay=([^,]+), status=(\S+)(.*)$/o) >= 4)
+		/to=<([^>]*)>, (?:orig_to=<[^>]*>, )?relay=([^,]+), (?:conn_use=[^,]+, )?delay=([^,]+), (?:delays=[^,]+, )?(?:dsn=[^,]+, )?status=(\S+)(.*)$/o) >= 4)
 	{
 
 	    if($opts{'m'} && $addr =~ /^(.*!)*([^!]+)!([^!@]+)@([^\.]+)$/o) {
@@ -739,13 +763,15 @@ while(<>) {
 		}
 		push(@{$msgDetail{$qid}}, $addr) if($opts{'e'});
 	    } elsif($status eq 'deferred') {
-		my ($deferredReas) = $logRmdr =~ /, status=deferred \(([^\)]+)/o;
-		unless(defined($opts{'verbMsgDetail'})) {
-		    $deferredReas = said_string_trimmer($deferredReas, 65);
-		    $deferredReas =~ s/^\d{3} //o;
-		    $deferredReas =~ s/^connect to //o;
+		unless($opts{'noDeferralDetail'}) {
+		    my ($deferredReas) = $logRmdr =~ /, status=deferred \(([^\)]+)/o;
+		    unless(defined($opts{'verbMsgDetail'})) {
+			$deferredReas = said_string_trimmer($deferredReas, 65);
+			$deferredReas =~ s/^\d{3} //o;
+			$deferredReas =~ s/^connect to //o;
+		    }
+		    ++$deferred{$cmd}{$deferredReas};
 		}
-		++$deferred{$cmd}{$deferredReas};
                 ++$dfrPerHr[$msgHr];
 		++${$msgsPerDay{$revMsgDateStr}}[2];
 		++$msgsDfrdCnt;
@@ -757,12 +783,14 @@ while(<>) {
 		    ${$recipDom{$domAddr}}[$msgDlyMaxI] = $delay
 		}
 	    } elsif($status eq 'bounced') {
-		my ($bounceReas) = $logRmdr =~ /, status=bounced \((.+)\)/o;
-		unless(defined($opts{'verbMsgDetail'})) {
-		    $bounceReas = said_string_trimmer($bounceReas, 66);
-		    $bounceReas =~ s/^\d{3} //o;
+		unless($opts{'noBounceDetail'}) {
+		    my ($bounceReas) = $logRmdr =~ /, status=bounced \((.+)\)/o;
+		    unless(defined($opts{'verbMsgDetail'})) {
+			$bounceReas = said_string_trimmer($bounceReas, 66);
+			$bounceReas =~ s/^\d{3} //o;
+		    }
+		    ++$bounced{$relay}{$bounceReas};
 		}
-		++$bounced{$relay}{$bounceReas};
                 ++$bncPerHr[$msgHr];
 		++${$msgsPerDay{$revMsgDateStr}}[3];
 		++$msgsBncd;
@@ -833,7 +861,6 @@ printf " %6d%s  sending hosts/domains\n", adj_int_units($sendgDomCnt);
 printf " %6d%s  recipients\n", adj_int_units($recipUserCnt);
 printf " %6d%s  recipient hosts/domains\n", adj_int_units($recipDomCnt);
 
-# ---Begin: SMTPD_STATS_SUPPORT---
 if(defined($opts{'smtpdStats'})) {
     print "\nsmtpd\n\n";
     printf "  %6d%s  connections\n", adj_int_units($smtpdConnCnt);
@@ -846,7 +873,6 @@ if(defined($opts{'smtpdStats'})) {
 	  $hr, $min, $sec;
     }
 }
-# ---End: SMTPD_STATS_SUPPORT---
 
 print "\n";
 
@@ -859,13 +885,11 @@ print_per_hour_summary(\@rcvPerHr, \@dlvPerHr, \@dfrPerHr, \@bncPerHr,
 print_recip_domain_summary(\%recipDom, $opts{'h'});
 print_sending_domain_summary(\%sendgDom, $opts{'h'});
 
-# ---Begin: SMTPD_STATS_SUPPORT---
 if(defined($opts{'smtpdStats'})) {
     print_per_day_smtpd(\%smtpdPerDay, $dayCnt) if($dayCnt > 1);
     print_per_hour_smtpd(\@smtpdPerHr, $dayCnt);
     print_domain_smtpd_summary(\%smtpdPerDom, $opts{'h'});
 }
-# ---End: SMTPD_STATS_SUPPORT---
 
 print_user_data(\%sendgUser, "Senders by message count", $msgCntI, $opts{'u'}, $opts{'q'});
 print_user_data(\%recipUser, "Recipients by message count", $msgCntI, $opts{'u'}, $opts{'q'});
@@ -894,8 +918,8 @@ sub print_problems_reports {
     }
     print_nested_hash(\%smtpMsgs, "smtp delivery failures", $opts{'q'});
     print_nested_hash(\%warnings, "Warnings", $opts{'q'});
-    print_nested_hash(\%fatals, "Fatal Errors", 0, $opts{'q'});
-    print_nested_hash(\%panics, "Panics", 0, $opts{'q'});
+    print_nested_hash(\%fatals, "Fatal Errors", $opts{'q'});
+    print_nested_hash(\%panics, "Panics", $opts{'q'});
     print_hash_by_cnt_vals(\%masterMsgs,"Master daemon messages", 0, $opts{'q'});
 }
 
@@ -1048,7 +1072,6 @@ sub print_user_data {
     }
 }
 
-# ---Begin: SMTPD_STATS_SUPPORT---
 
 # print "per-hour" smtpd connection summary
 # (done in a subroutine only to keep main-line code clean)
@@ -1100,7 +1123,6 @@ End_Of_Per_Hour_Smtp
     }
 }
 
-
 # print "per-day" smtpd connection summary
 # (done in a subroutine only to keep main-line code clean)
 sub print_per_day_smtpd {
@@ -1162,8 +1184,6 @@ End_Of_Domain_Smtp_Heading
     }
 }
 
-# ---End: SMTPD_STATS_SUPPORT---
-
 # print hash contents sorted by numeric values in descending
 # order (i.e.: highest first)
 sub print_hash_by_cnt_vals {
@@ -1489,6 +1509,14 @@ sub proc_smtpd_reject {
     my ($from, $to);
     my $rejAddFrom = 0;
 
+    ++$$msgsRjctd;
+    ++$$rejPerHr;
+    ++$$msgsPerDay;
+
+    # Hate the sub-calling overhead if we're not doing reject details
+    # anyway, but this is the only place we can do this.
+    return if($opts{'noRejectDetail'});
+
     # This could get real ugly!
 
     # First: get everything following the "reject: ", etc. token
@@ -1565,9 +1593,6 @@ sub proc_smtpd_reject {
 	$rejData .= "  ($from)" if($rejAddFrom);
 	++$rejects->{$rejTyp}{$rejReas}{$rejData};
     }
-    ++$$msgsRjctd;
-    ++$$rejPerHr;
-    ++$$msgsPerDay;
 }
 
 # Hack for VERP (?) - convert address from somthing like
